stopwords = {
    'а',
    'алло',
    'без',
    'близко',
    'более',
    'больше',
    'будем',
    'будет',
    'будете',
    'будешь',
    'будто',
    'буду',
    'будут',
    'будь',
    'бы',
    'бывает',
    'бывь',
    'был',
    'была',
    'были',
    'было',
    'быть',
    'в',
    'важная',
    'важное',
    'важные',
    'важный',
    'вам',
    'вами',
    'вас',
    'ваш',
    'ваша',
    'ваше',
    'ваши',
    'вверх',
    'вдали',
    'вдруг',
    'ведь',
    'везде',
    'весь',
    'вниз',
    'внизу',
    'во',
    'вокруг',
    'вон',
    'восемнадцатый',
    'восемнадцать',
    'восемь',
    'восьмой',
    'вот',
    'впрочем',
    'времени',
    'время',
    'все',
    'всегда',
    'всего',
    'всем',
    'всеми',
    'всему',
    'всех',
    'всею',
    'всю',
    'всюду',
    'вся',
    'всё',
    'второй',
    'вы',
    'г   год',
    'где',
    'говорил',
    'говорит',
    'года',
    'году',
    'да',
    'давно',
    'даже',
    'далеко',
    'дальше',
    'даром',
    'два',
    'двадцатый',
    'двадцать',
    'две',
    'двенадцатый',
    'двенадцать',
    'двух',
    'девятнадцатый',
    'девятнадцать',
    'девятый',
    'девять',
    'действительно',
    'дел',
    'день',
    'десятый',
    'десять',
    'для',
    'до',
    'довольно',
    'долго',
    'должно',
    'другая',
    'другие',
    'других',
    'друго',
    'другое',
    'другой',
    'е',
    'его',
    'ее',
    'ей',
    'ему',
    'если',
    'есть',
    'еще',
    'ещё',
    'ею',
    'её',
    'ж',
    'же',
    'жизнь',
    'за',
    'занят',
    'занята',
    'занято',
    'заняты',
    'затем',
    'зато',
    'зачем',
    'здесь',
    'значит',
    'и',
    'из',
    'или',
    'им',
    'именно',
    'иметь',
    'ими',
    'имя',
    'иногда',
    'их',
    'к',
    'каждая',
    'каждое',
    'каждые',
    'каждый',
    'кажется',
    'как',
    'какая',
    'какой',
    'кем',
    'когда',
    'кого',
    'ком',
    'кому',
    'конечно',
    'которая',
    'которого',
    'которой',
    'которые',
    'который',
    'которых',
    'кроме',
    'кругом',
    'кто',
    'куда',
    'лет',
    'ли',
    'лишь',
    'лучше',
    'люди',
    'м',
    'мало',
    'между',
    'меля',
    'менее',
    'меньше',
    'меня',
    'миллионов',
    'мимо',
    'мира',
    'мне',
    'много',
    'многочисленная',
    'многочисленное',
    'многочисленные',
    'многочисленный',
    'мной',
    'мною',
    'мог',
    'могут',
    'мож',
    'может',
    'можно',
    'можхо',
    'мои',
    'мой',
    'мор',
    'мочь',
    'моя',
    'моё',
    'мы',
    'на',
    'наверху',
    'над',
    'надо',
    'назад',
    'наиболее',
    'наконец',
    'нам',
    'нами',
    'нас',
    'начала',
    'наш',
    'наша',
    'наше',
    'наши',
    'не',
    'него',
    'недавно',
    'недалеко',
    'нее',
    'ней',
    'нельзя',
    'нем',
    'немного',
    'нему',
    'непрерывно',
    'нередко',
    'несколько',
    'нет',
    'нею',
    'неё',
    'ни',
    'нибудь',
    'ниже',
    'низко',
    'никогда',
    'никуда',
    'ними',
    'них',
    'ничего',
    'но',
    'ну',
    'нужно',
    'нх',
    'о',
    'об',
    'оба',
    'обычно',
    'один',
    'одиннадцатый',
    'одиннадцать',
    'однажды',
    'однако',
    'одного',
    'одной',
    'около',
    'он',
    'она',
    'они',
    'оно',
    'опять',
    'особенно',
    'от',
    'отовсюду',
    'отсюда',
    'очень',
    'первый',
    'перед',
    'по',
    'под',
    'пожалуйста',
    'позже',
    'пока',
    'пор',
    'пора',
    'после',
    'посреди',
    'потом',
    'потому',
    'почему',
    'почти',
    'прекрасно',
    'при',
    'про',
    'просто',
    'против',
    'процентов',
    'пятнадцатый',
    'пятнадцать',
    'пятый',
    'пять',
    'с',
}



stopwords = {
    'a',
    'about',
    'above',
    'after',
    'again',
    'against',
    'all',
    'am',
    'an',
    'and',
    'any',
    'are',
    'aren',
    'as',
    'at',
    'be',
    'because',
    'been',
    'before',
    'being',
    'below',
    'between',
    'both',
    'but',
    'by',
    'can',
    'cannot',
    'could',
    'couldn',
    'd',
    'did',
    'didn',
    'do',
    'does',
    'doesn',
    'doing',
    'don',
    'down',
    'during',
    'each',
    'few',
    'for',
    'from',
    'further',
    'had',
    'hadn',
    'has',
    'hasn',
    'have',
    'haven',
    'having',
    'he',
    'her',
    'here',
    'hers',
    'herself',
    'him',
    'himself',
    'his',
    'how',
    'i',
    'if',
    'in',
    'into',
    'is',
    'isn',
    'it',
    'its',
    'itself',
    'let',
    'll',
    'm',
    'me',
    'more',
    'most',
    'mustn',
    'my',
    'myself',
    'no',
    'nor',
    'not',
    'of',
    'off',
    'on',
    'once',
    'only',
    'or',
    'other',
    'ought',
    'our',
    'ours',
    'ourselves',
    'out',
    'over',
    'own',
    're',
    's',
    'same',
    'shan',
    'she',
    'should',
    'shouldn',
    'so',
    'some',
    'such',
    't',
    'than',
    'that',
    'the',
    'their',
    'theirs',
    'them',
    'themselves',
    'then',
    'there',
    'these',
    'they',
    'this',
    'those',
    'through',
    'to',
    'too',
    'under',
    'until',
    'up',
    've',
    'very',
    'was',
    'wasn',
    'we',
    'were',
    'weren',
    'what',
    'when',
    'where',
    'which',
    'while',
    'who',
    'whom',
    'why',
    'with',
    'won',
    'would',
    'wouldn',
    'you',
    'your',
    'yours',
    'yourself',
    'yourselves',
}



import pkgutil
import importlib
import os.path as op

from Stemmer import Stemmer
import regex as re

import tldry.stopwords as stopwords


stopwords_dirname = op.dirname(stopwords.__file__)
AVAILABLE_LANGUAGES = frozenset(
    name for _, name, _ in pkgutil.iter_modules([stopwords_dirname]))


class TextCleaner:
    word_regex = re.compile(r'[^\W\d_]+')
    sent_regex = re.compile(r'(?<=(?:[\p{lower}\d]|[^\w])\s*[.!?\n]+)'
                            r'\s*(?=(?:[\p{upper}\d]|[^\w]))')

    def __init__(self, language, min_sent_len):
        if language not in AVAILABLE_LANGUAGES:
            err = (f"Language '{language}' is not available, "
                   f"choose from [{', '.join(AVAILABLE_LANGUAGES)}]")
            raise ValueError(err)

        self.min_sent_len = min_sent_len

        self.stem = Stemmer(language).stemWord

        stopwords = importlib.import_module(f'tldry.stopwords.{language}')
        self.stopwords = frozenset(stopwords.stopwords)

        self.raw_sentences = []

    def fit_transform(self, text):
        raw_sentences_ = self.sent_tokenize(text)
        clean_sentences_ = [self.clean_sentence(self.word_tokenize(sent))
                            for sent in raw_sentences_]

        used_sentences = set()
        raw_sentences = []
        clean_sentences = []
        for c, r in zip(clean_sentences_, raw_sentences_):
            c = tuple(c)
            if len(c) < self.min_sent_len or c in used_sentences:
                continue
            clean_sentences.append(c)
            raw_sentences.append(r)
            used_sentences.add(c)

        self.raw_sentences = raw_sentences

        return clean_sentences

    def sent_tokenize(self, text):
        sentences = []
        for line in text.splitlines():
            for sent in self.sent_regex.split(line.strip()):
                sentences.append(sent)
        return sentences

    def word_tokenize(self, sentence):
        return self.word_regex.findall(sentence)

    def clean_sentence(self, sentence):
        new_sentence = []
        for tok in sentence:
            tok = tok.lower()
            if tok in self.stopwords or len(tok) == 1:
                continue
            new_sentence.append(self.stem(tok))

        return new_sentence



from collections import defaultdict

import numpy as np


class PDistMaker:

    def __init__(self, min_df=2):
        self.min_df = min_df

        self.vocabulary_ = {}

    def transform_tf(self, corpus):
        corpus_size = len(corpus)

        tf_matrix = np.zeros((corpus_size, len(self.vocabulary_)))
        for sent_id, kws in enumerate(corpus):
            for kw in kws:
                kw_id = self.vocabulary_.get(kw)
                if kw_id is not None:
                    tf_matrix[sent_id, kw_id] += 1

        return tf_matrix

    def fit_transform(self, corpus):
        self.fit(corpus)
        return self.transform(corpus)

    def transform(self, corpus):
        tf_matrix = self.transform_tf(corpus)
        sent_sizes = np.asarray([len(sent) for sent in corpus])[:, None]

        sizes_matrix = np.log(sent_sizes.dot(sent_sizes.T))
        sizes_matrix[sizes_matrix == 0] = 1

        distance_matrix = np.inner(tf_matrix, tf_matrix)

        return distance_matrix / sizes_matrix

    def fit(self, corpus):
        corpus_size = len(corpus)
        self.vocabulary_ = {}

        keywords_df = defaultdict(int)

        for kws in corpus:
            for kw in set(kws):
                keywords_df[kw] += 1

        kw2df = [(kw, df) for kw, df in keywords_df.items()
                 if df >= self.min_df]

        for kw_id, (kw, df) in enumerate(kw2df):
            self.vocabulary_[kw] = kw_id

        return self



#!/usr/bin/env python
from setuptools import setup, find_packages


requirements = [
    'numpy',
    'regex',
    'PyStemmer==1.3.0',
]

setup(
    name='tldry',
    version='0.3.1',
    author='Artem Vang',
    author_email='vangogius@gmail.com',
    url='https://github.com/vangaa/tldry',
    description='Automatic text summarizer based on textrank algorithm',
    license='MIT License',
    install_requires=requirements,
    packages=find_packages('.', exclude='tests'),
    classifiers=(
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: Implementation :: CPython',
    )
)



import numpy as np

from tldry.nlp import TextCleaner
from tldry.pdist_maker import PDistMaker


DEFAULT_LANGUAGE = 'english'


class TLDRy:
    damping = 0.85

    def __init__(self, language=DEFAULT_LANGUAGE,
                 min_sent_len=2, min_df=2):
        self.language = language
        self.text_cleaner = TextCleaner(
            language=language,
            min_sent_len=min_sent_len,
        )
        self.dist_matrix_maker = PDistMaker(
            min_df=min_df,
        )

    def summarize(self, text, topn=5, with_scores=False,
                  sort_by_position=True):
        clean_text = self.text_cleaner.fit_transform(text)
        raw_sentences = self.text_cleaner.raw_sentences

        if not raw_sentences:
            return []

        distance_matrix = self.dist_matrix_maker.fit_transform(clean_text)
        fitted_distance_matrix = self.preprocess_distance_matrix(
            distance_matrix)

        sentence_scores = self.textrank(fitted_distance_matrix)

        argsorted_ids = np.argsort(sentence_scores)[-topn:][::-1]
        if sort_by_position:
            argsorted_ids = sorted(argsorted_ids)

        topn_sentences = [raw_sentences[i] for i in argsorted_ids]

        if with_scores:
            scores = sentence_scores[argsorted_ids]
            return topn_sentences, scores

        return topn_sentences

    def preprocess_distance_matrix(self, distance_matrix):
        sums = distance_matrix.sum(axis=1)
        sums[sums == 0] = 1
        distance_matrix /= sums[:, None]

        return distance_matrix

    def textrank(self, distance_matrix):
        sentences_count = len(distance_matrix)

        basic_probability_matrix = ((1 - self.damping) *
                                    np.ones_like(distance_matrix) /
                                    sentences_count)
        matrix = self.damping * distance_matrix.T
        matrix += basic_probability_matrix

        eig_vector = np.ones((sentences_count, )) / sentences_count
        error_val = 1.0

        while error_val > 1e-4:
            next_eig = np.dot(matrix, eig_vector)
            error_val = np.linalg.norm(next_eig - eig_vector)
            eig_vector = next_eig

        return eig_vector
